@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2004-2006 William Jon McCann <mccann@jhu.edu>
+ * Copyright (C) 2004-2005 William Jon McCann <mccann@jhu.edu>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -53,7 +53,7 @@ struct GSManagerPrivate
         guint        lock_enabled : 1;
         guint        logout_enabled : 1;
         guint        user_switch_enabled : 1;
-        guint        throttle_enabled : 1;
+        guint        throttled : 1;
 
         char        *logout_command;
 
@@ -61,7 +61,6 @@ struct GSManagerPrivate
         guint        active : 1;
         guint        lock_active : 1;
 
-        guint        fading : 1;
         guint        dialog_up : 1;
 
         guint        lock_timeout_id;
@@ -71,8 +70,6 @@ struct GSManagerPrivate
 
         GSList      *themes;
         GSSaverMode  saver_mode;
-        GSFade      *fade;
-        guint        unfade_idle_id;
 };
 
 enum {
@@ -91,7 +88,7 @@ enum {
         PROP_LOGOUT_TIMEOUT,
         PROP_LOGOUT_COMMAND,
         PROP_ACTIVE,
-        PROP_THROTTLE_ENABLED,
+        PROP_THROTTLED,
 };
 
 #define FADE_TIMEOUT 1000
@@ -138,71 +135,6 @@ select_theme (GSManager *manager)
         return theme;
 }
 
-
-static GSJob *
-lookup_job_for_window (GSManager *manager,
-                       GSWindow  *window)
-{
-        GSJob *job;
-
-        if (manager->priv->jobs == NULL) {
-                return NULL;
-        }
-
-        job = g_hash_table_lookup (manager->priv->jobs, window);
-
-        return job;
-}
-
-static void
-manager_maybe_stop_job_for_window (GSManager *manager,
-                                   GSWindow  *window)
-{
-        GSJob *job;
-
-        job = lookup_job_for_window (manager, window);
-
-        if (job == NULL) {
-                gs_debug ("Job not found for window");
-                return;
-        }
-
-        gs_job_stop (job);
-}
-
-static void
-manager_maybe_start_job_for_window (GSManager *manager,
-                                    GSWindow  *window)
-{
-        GSJob *job;
-
-        job = lookup_job_for_window (manager, window);
-
-        if (job == NULL) {
-                gs_debug ("Job not found for window");
-                return;
-        }
-
-        if (! manager->priv->dialog_up) {
-                if (! manager->priv->throttle_enabled) {
-                        if (! gs_job_is_running (job)) {
-                                if (! gs_window_is_obscured (window)) {
-                                        gs_debug ("Starting job for window");
-                                        gs_job_start (job);
-                                } else {
-                                        gs_debug ("Window is obscured deferring start of job");
-                                }
-                        } else {
-                                gs_debug ("Not starting job because job is running");
-                        }
-                } else {
-                        gs_debug ("Not starting job because throttled");
-                }
-        } else {
-                gs_debug ("Not starting job because dialog is up");
-        }
-}
-
 static void
 cycle_job (GSWindow  *window,
            GSJob     *job,
@@ -214,7 +146,7 @@ cycle_job (GSWindow  *window,
 
         gs_job_stop (job);
         gs_job_set_theme (job, theme, NULL);
-        manager_maybe_start_job_for_window (manager, window);
+        gs_job_start (job);
 }
 
 static void
@@ -230,10 +162,10 @@ throttle_job (GSWindow  *window,
               GSJob     *job,
               GSManager *manager)
 {
-        if (manager->priv->throttle_enabled) {
+        if (manager->priv->throttled) {
                 gs_job_stop (job);
         } else {
-                manager_maybe_start_job_for_window (manager, window);
+                gs_job_start (job);
         }
 }
 
@@ -253,7 +185,7 @@ resume_job (GSWindow  *window,
         if (gs_job_is_running (job)) {
                 gs_job_suspend (job, FALSE);
         } else {
-                manager_maybe_start_job_for_window (manager, window);
+                gs_job_start (job);
         }
 }
 
@@ -320,15 +252,15 @@ gs_manager_set_themes (GSManager *manager,
 }
 
 void
-gs_manager_set_throttle_enabled (GSManager *manager,
-                                 gboolean   throttle_enabled)
+gs_manager_set_throttled (GSManager *manager,
+                          gboolean   throttled)
 {
         g_return_if_fail (GS_IS_MANAGER (manager));
 
-        if (manager->priv->throttle_enabled != throttle_enabled) {
+        if (manager->priv->throttled != throttled) {
                 GSList *l;
 
-                manager->priv->throttle_enabled = throttle_enabled;
+                manager->priv->throttled = throttled;
 
                 if (! manager->priv->dialog_up) {
 
@@ -341,27 +273,12 @@ gs_manager_set_throttle_enabled (GSManager *manager,
         }
 }
 
-void
-gs_manager_get_lock_active (GSManager *manager,
-                            gboolean  *lock_active)
-{
-        if (lock_active != NULL) {
-                *lock_active = FALSE;
-        }
-
-        g_return_if_fail (GS_IS_MANAGER (manager));
-
-        *lock_active = manager->priv->lock_active;
-}
-
 void
 gs_manager_set_lock_active (GSManager *manager,
                             gboolean   lock_active)
 {
         g_return_if_fail (GS_IS_MANAGER (manager));
 
-        gs_debug ("Setting lock active: %d", lock_active);
-
         if (manager->priv->lock_active != lock_active) {
                 GSList *l;
 
@@ -418,9 +335,8 @@ gs_manager_set_user_switch_enabled (GSManager *manager,
 static gboolean
 activate_lock_timeout (GSManager *manager)
 {
-        if (manager->priv->lock_enabled) {
+        if (manager->priv->lock_enabled)
                 gs_manager_set_lock_active (manager, TRUE);
-        }
 
         manager->priv->lock_timeout_id = 0;
 
@@ -509,8 +425,6 @@ gs_manager_cycle (GSManager *manager)
         g_return_val_if_fail (manager != NULL, FALSE);
         g_return_val_if_fail (GS_IS_MANAGER (manager), FALSE);
 
-        gs_debug ("cycling jobs");
-
         if (! manager->priv->active) {
                 return FALSE;
         }
@@ -519,7 +433,7 @@ gs_manager_cycle (GSManager *manager)
                 return FALSE;
         }
 
-        if (manager->priv->throttle_enabled) {
+        if (manager->priv->throttled) {
                 return FALSE;
         }
 
@@ -598,8 +512,8 @@ gs_manager_set_property (GObject            *object,
         self = GS_MANAGER (object);
 
         switch (prop_id) {
-        case PROP_THROTTLE_ENABLED:
-                gs_manager_set_throttle_enabled (self, g_value_get_boolean (value));
+        case PROP_THROTTLED:
+                gs_manager_set_throttled (self, g_value_get_boolean (value));
                 break;
         case PROP_LOCK_ENABLED:
                 gs_manager_set_lock_enabled (self, g_value_get_boolean (value));
@@ -639,8 +553,8 @@ gs_manager_get_property (GObject            *object,
         self = GS_MANAGER (object);
 
         switch (prop_id) {
-        case PROP_THROTTLE_ENABLED:
-                g_value_set_boolean (value, self->priv->throttle_enabled);
+        case PROP_THROTTLED:
+                g_value_set_boolean (value, self->priv->throttled);
                 break;
         case PROP_LOCK_ENABLED:
                 g_value_set_boolean (value, self->priv->lock_enabled);
@@ -767,8 +681,8 @@ gs_manager_class_init (GSManagerClass *klass)
                                                             300000,
                                                             G_PARAM_READWRITE));
         g_object_class_install_property (object_class,
-                                         PROP_THROTTLE_ENABLED,
-                                         g_param_spec_boolean ("throttle-enabled",
+                                         PROP_THROTTLED,
+                                         g_param_spec_boolean ("throttled",
                                                                NULL,
                                                                NULL,
                                                                TRUE,
@@ -781,8 +695,6 @@ static void
 gs_manager_init (GSManager *manager)
 {
         manager->priv = GS_MANAGER_GET_PRIVATE (manager);
-
-        manager->priv->fade = gs_fade_new ();
 }
 
 static void
@@ -823,8 +735,6 @@ gs_manager_finalize (GObject *object)
         manager->priv->activate_time = 0;
         manager->priv->lock_enabled = FALSE;
 
-        g_object_unref (manager->priv->fade);
-
         G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -863,12 +773,6 @@ window_dialog_up_cb (GSWindow  *window,
         gs_debug ("Handling dialog up");
 
         manager->priv->dialog_up = TRUE;
-        /* Make all other windows insensitive so we don't get events */
-        for (l = manager->priv->windows; l; l = l->next) {
-                if (l->data != window) {
-                        gtk_widget_set_sensitive (GTK_WIDGET (l->data), FALSE);
-                }
-        }
 
         /* Move keyboard and mouse grabs so dialog can be used */
         gs_grab_window (gs_window_get_gdk_window (window),
@@ -879,7 +783,14 @@ window_dialog_up_cb (GSWindow  *window,
            the dialog can be used.  We'll regrab it when the dialog goes down. */
         gs_grab_release_mouse ();
 
-        if (! manager->priv->throttle_enabled) {
+        /* Make all other windows insensitive so we don't get events */
+        for (l = manager->priv->windows; l; l = l->next) {
+                if (l->data != window) {
+                        gtk_widget_set_sensitive (GTK_WIDGET (l->data), FALSE);
+                }
+        }
+
+        if (! manager->priv->throttled) {
                 gs_debug ("Suspending jobs");
 
                 manager_suspend_jobs (manager);
@@ -905,48 +816,33 @@ window_dialog_down_cb (GSWindow  *window,
                 gtk_widget_set_sensitive (GTK_WIDGET (l->data), TRUE);
         }
 
-        manager->priv->dialog_up = FALSE;
-
-        if (! manager->priv->throttle_enabled) {
+        if (! manager->priv->throttled) {
                 manager_resume_jobs (manager);
         }
+
+        manager->priv->dialog_up = FALSE;
 }
 
-static GSWindow *
-find_window_at_pointer (GSManager *manager)
+static void
+manager_maybe_start_job_for_window (GSManager *manager,
+                                    GSWindow  *window)
 {
-        GdkDisplay *display;
-        GdkScreen  *screen;
-        int         monitor;
-        int         x, y;
-        GSWindow   *window;
-        int         screen_num;
-        GSList     *l;
+        GSJob *job;
 
-        display = gdk_display_get_default ();
-        gdk_display_get_pointer (display, &screen, &x, &y, NULL);
-        monitor = gdk_screen_get_monitor_at_point (screen, x, y);
-        screen_num = gdk_screen_get_number (screen);
-
-        /* Find the gs-window that is on that screen */
-        window = NULL;
-        for (l = manager->priv->windows; l; l = l->next) {
-                GSWindow *win = GS_WINDOW (l->data);
-                if (gs_window_get_screen (win) == screen
-                    && gs_window_get_monitor (win) == monitor) {
-                        window = win;
-                }
+        if (manager->priv->jobs == NULL) {
+                return;
         }
 
-        if (window == NULL) {
-                gs_debug ("WARNING: Could not find the GSWindow for screen %d", screen_num);
-                /* take the first one */
-                window = manager->priv->windows->data;
-        } else {
-                gs_debug ("Requesting unlock for screen %d", screen_num);
+        job = g_hash_table_lookup (manager->priv->jobs, window);
+
+        if (job == NULL) {
+                gs_debug ("Job not found for window");
+                return;
         }
 
-        return window;
+        if (! manager->priv->throttled) {
+                gs_job_start (job);
+        }
 }
 
 static gboolean
@@ -989,32 +885,6 @@ window_grab_broken_cb (GSWindow           *window,
         }
 }
 
-static gboolean
-unfade_idle (GSManager *manager)
-{
-        gs_debug ("resetting fade");
-        gs_fade_reset (manager->priv->fade);
-        manager->priv->unfade_idle_id = 0;
-        return FALSE;
-}
-
-
-static void
-remove_unfade_idle (GSManager *manager)
-{
-        if (manager->priv->unfade_idle_id > 0) {
-                g_source_remove (manager->priv->unfade_idle_id);
-                manager->priv->unfade_idle_id = 0;
-        }
-}
-
-static void
-add_unfade_idle (GSManager *manager)
-{
-        remove_unfade_idle (manager);
-        manager->priv->unfade_idle_id = g_idle_add ((GSourceFunc)unfade_idle, manager);
-}
-
 static gboolean
 window_map_event_cb (GSWindow  *window,
                      GdkEvent  *event,
@@ -1059,6 +929,9 @@ manager_show_window (GSManager *manager,
 
         manager->priv->activate_time = time (NULL);
 
+        /* Set lock state to off initially.  It may be enabled by activate_lock_timout() */
+        gs_manager_set_lock_active (manager, FALSE);
+
         if (manager->priv->lock_timeout >= 0) {
                 remove_lock_timer (manager);
                 add_lock_timer (manager, manager->priv->lock_timeout);
@@ -1069,8 +942,6 @@ manager_show_window (GSManager *manager,
                 add_cycle_timer (manager, manager->priv->cycle_timeout);
         }
 
-        add_unfade_idle (manager);
-
         /* FIXME: only emit signal once */
         g_signal_emit (manager, signals [ACTIVATED], 0);
 }
@@ -1089,58 +960,16 @@ window_show_cb (GSWindow  *window,
         manager_show_window (manager, window);
 }
 
-static gboolean
-window_activity_cb (GSWindow  *window,
-                    GSManager *manager)
-{
-        gboolean handled;
-
-        handled = gs_manager_request_unlock (manager);
-
-        return handled;
-}
-
-static void
-maybe_set_window_throttle (GSManager *manager,
-                           GSWindow  *window,
-                           gboolean   throttled)
-{
-        if (throttled) {
-                manager_maybe_stop_job_for_window (manager, window);
-        } else {
-                manager_maybe_start_job_for_window (manager, window);
-        }
-}
-
-static void
-window_obscured_cb (GSWindow   *window,
-                    GParamSpec *pspec,
-                    GSManager  *manager)
-{
-        gboolean obscured;
-
-        obscured = gs_window_is_obscured (window);
-        gs_debug ("Handling window obscured: %s", obscured ? "obscured" : "unobscured");
-
-        maybe_set_window_throttle (manager, window, obscured);
-
-        if (! obscured) {
-                gs_manager_request_unlock (manager);
-        }
-}
-
 static void
 disconnect_window_signals (GSManager *manager,
                            GSWindow  *window)
 {
         g_signal_handlers_disconnect_by_func (window, window_deactivated_cb, manager);
-        g_signal_handlers_disconnect_by_func (window, window_activity_cb, manager);
         g_signal_handlers_disconnect_by_func (window, window_dialog_up_cb, manager);
         g_signal_handlers_disconnect_by_func (window, window_dialog_down_cb, manager);
         g_signal_handlers_disconnect_by_func (window, window_show_cb, manager);
         g_signal_handlers_disconnect_by_func (window, window_map_cb, manager);
         g_signal_handlers_disconnect_by_func (window, window_map_event_cb, manager);
-        g_signal_handlers_disconnect_by_func (window, window_obscured_cb, manager);
         g_signal_handlers_disconnect_by_func (window, window_unmap_cb, manager);
         g_signal_handlers_disconnect_by_func (window, window_grab_broken_cb, manager);
 }
@@ -1158,8 +987,6 @@ connect_window_signals (GSManager *manager,
 {
         g_signal_connect_object (window, "destroy",
                                  G_CALLBACK (window_destroyed_cb), manager, 0);
-        g_signal_connect_object (window, "activity",
-                                 G_CALLBACK (window_activity_cb), manager, 0);
         g_signal_connect_object (window, "deactivated",
                                  G_CALLBACK (window_deactivated_cb), manager, 0);
         g_signal_connect_object (window, "dialog-up",
@@ -1172,8 +999,6 @@ connect_window_signals (GSManager *manager,
                                  G_CALLBACK (window_map_cb), manager, G_CONNECT_AFTER);
         g_signal_connect_object (window, "map_event",
                                  G_CALLBACK (window_map_event_cb), manager, G_CONNECT_AFTER);
-        g_signal_connect_object (window, "notify::obscured",
-                                 G_CALLBACK (window_obscured_cb), manager, G_CONNECT_AFTER);
         g_signal_connect_object (window, "unmap",
                                  G_CALLBACK (window_unmap_cb), manager, G_CONNECT_AFTER);
         g_signal_connect_object (window, "grab_broken_event",
@@ -1269,15 +1094,6 @@ remove_job (GSJob *job)
         g_object_unref (job);
 }
 
-static void
-fade_done_cb (GSFade    *fade,
-              GSManager *manager)
-{
-        gs_debug ("fade completed, showing windows");
-        show_windows (manager->priv->windows);
-        manager->priv->fading = FALSE;
-}
-
 static gboolean
 gs_manager_activate (GSManager *manager)
 {
@@ -1316,16 +1132,15 @@ gs_manager_activate (GSManager *manager)
         /* fade to black and show windows */
         do_fade = TRUE;
         if (do_fade) {
-                manager->priv->fading = TRUE;
-                gs_debug ("fading out");
-                gs_fade_async (manager->priv->fade,
-                               FADE_TIMEOUT,
-                               (GSFadeDoneFunc)fade_done_cb,
-                               manager);
+                GSFade *fade;
 
-                while (manager->priv->fading) {
-                        gtk_main_iteration ();
-                }
+                fade = gs_fade_new ();
+                gs_fade_set_timeout (fade, FADE_TIMEOUT);
+                gs_fade_now (fade);
+                show_windows (manager->priv->windows);
+                g_usleep (500000);
+                gs_fade_reset (fade);
+                g_object_unref (fade);
         } else {
                 show_windows (manager->priv->windows);
         }
@@ -1361,6 +1176,7 @@ gs_manager_deactivate (GSManager *manager)
         manager->priv->active = FALSE;
         manager->priv->activate_time = 0;
         manager->priv->lock_active = FALSE;
+        manager->priv->throttled = FALSE;
 
         return TRUE;
 }
@@ -1389,37 +1205,32 @@ gs_manager_get_active (GSManager *manager)
         return manager->priv->active;
 }
 
-gboolean
+void
 gs_manager_request_unlock (GSManager *manager)
 {
-        GSWindow *window;
+        GSList     *l;
+        GdkDisplay *display;
+        GdkScreen  *screen;
+        int         screen_num;
 
-        g_return_val_if_fail (manager != NULL, FALSE);
-        g_return_val_if_fail (GS_IS_MANAGER (manager), FALSE);
+        g_return_if_fail (manager != NULL);
+        g_return_if_fail (GS_IS_MANAGER (manager));
 
         if (! manager->priv->active) {
-                gs_debug ("Request unlock but manager is not active");
-                return FALSE;
+                return;
         }
 
-        if (manager->priv->dialog_up) {
-                gs_debug ("Request unlock but dialog is already up");
-                return FALSE;
+        /* Find the screen that contains the pointer */
+        display = gdk_display_get_default ();
+        gdk_display_get_pointer (display, &screen, NULL, NULL, NULL);
+        screen_num = gdk_screen_get_number (screen);
+
+        /* Find the gs-window that is on that screen */
+        for (l = manager->priv->windows; l; l = l->next) {
+                int num = gdk_screen_get_number (GTK_WINDOW (l->data)->screen);
+                if (num == screen_num) {
+                        gs_window_request_unlock (GS_WINDOW (l->data));
+                        break;
+                }
         }
-
-        if (manager->priv->fading) {
-                gs_debug ("Request unlock so finishing fade");
-                gs_fade_finish (manager->priv->fade);
-        }
-
-        if (manager->priv->windows == NULL) {
-                gs_debug ("We don't have any windows!");
-                return FALSE;
-        }
-
-        /* Find the GSWindow that contains the pointer */
-        window = find_window_at_pointer (manager);
-        gs_window_request_unlock (window);
-
-        return TRUE;
 }
