@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2004-2006 William Jon McCann <mccann@jhu.edu>
+ * Copyright (C) 2004-2005 William Jon McCann <mccann@jhu.edu>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@
 #include <gtk/gtk.h>
 
 #include "gs-window.h"
-#include "gs-marshal.h"
+#include "gs-visual-gl.h"
 #include "subprocs.h"
 #include "gs-debug.h"
 
@@ -41,6 +41,7 @@ static void gs_window_init       (GSWindow      *window);
 static void gs_window_finalize   (GObject       *object);
 
 static gboolean popup_dialog_idle (GSWindow *window);
+static gboolean gs_window_request_unlock_idle (GSWindow *window);
 
 enum {
         DIALOG_RESPONSE_CANCEL,
@@ -54,7 +55,6 @@ struct GSWindowPrivate
         int        monitor;
 
         GdkRectangle geometry;
-        guint      obscured : 1;
 
         guint      lock_enabled : 1;
         guint      user_switch_enabled : 1;
@@ -65,6 +65,7 @@ struct GSWindowPrivate
         GtkWidget *box;
         GtkWidget *socket;
 
+        guint      request_unlock_idle_id;
         guint      popup_dialog_idle_id;
 
         guint      dialog_map_signal_id;
@@ -86,7 +87,6 @@ struct GSWindowPrivate
 };
 
 enum {
-        ACTIVITY,
         DEACTIVATED,
         DIALOG_UP,
         DIALOG_DOWN,
@@ -95,7 +95,6 @@ enum {
 
 enum {
         PROP_0,
-        PROP_OBSCURED,
         PROP_LOCK_ENABLED,
         PROP_LOGOUT_ENABLED,
         PROP_LOGOUT_COMMAND,
@@ -163,32 +162,6 @@ gs_window_override_user_time (GSWindow *window)
         gdk_x11_window_set_user_time (GTK_WIDGET (window)->window, ev_time);
 }
 
-static void
-force_no_pixmap_background (GtkWidget *widget)
-{
-        static gboolean first_time = TRUE;
-
-        if (first_time) {
-                gtk_rc_parse_string ("\n"
-                                     "   style \"gs-theme-engine-style\"\n"
-                                     "   {\n"
-                                     "      bg_pixmap[NORMAL] = \"<none>\"\n"
-                                     "      bg_pixmap[INSENSITIVE] = \"<none>\"\n"
-                                     "      bg_pixmap[ACTIVE] = \"<none>\"\n"
-                                     "      bg_pixmap[PRELIGHT] = \"<none>\"\n"
-                                     "      bg[NORMAL] = { 0.0, 0.0, 0.0 }\n"
-                                     "      bg[INSENSITIVE] = { 0.0, 0.0, 0.0 }\n"
-                                     "      bg[ACTIVE] = { 0.0, 0.0, 0.0 }\n"
-                                     "      bg[PRELIGHT] = { 0.0, 0.0, 0.0 }\n"
-                                     "   }\n"
-                                     "   widget \"gs-window*\" style : highest \"gs-theme-engine-style\"\n"
-                                     "\n");
-                first_time = FALSE;
-        }
-
-        gtk_widget_set_name (widget, "gs-window");
-}
-
 static void
 clear_children (Window window)
 {
@@ -202,8 +175,9 @@ clear_children (Window window)
         status = XQueryTree (GDK_DISPLAY (), window, &root, &parent, &children, &n_children);
 
         if (status == 0) {
-                if (children)
+                if (children) {
                         XFree (children);
+                }
                 return;
         }
 
@@ -235,6 +209,7 @@ clear_all_children (GSWindow *window)
         clear_children (GDK_WINDOW_XID (w));
 
         gdk_display_sync (gtk_widget_get_display (GTK_WIDGET (window)));
+        gdk_flush ();
         gdk_error_trap_pop ();
 }
 
@@ -245,12 +220,6 @@ gs_window_clear (GSWindow *window)
         GdkColormap *colormap;
         GtkStateType state;
 
-        g_return_if_fail (GS_IS_WINDOW (window));
-
-        if (! GTK_WIDGET_VISIBLE (GTK_WIDGET (window))) {
-                return;
-        }
-
         gs_debug ("Clearing window");
 
         state = (GtkStateType) 0;
@@ -336,6 +305,8 @@ gs_window_real_unrealize (GtkWidget *widget)
 static void
 gs_window_real_realize (GtkWidget *widget)
 {
+        gs_visual_gl_widget_set_best_colormap (widget);
+
         if (GTK_WIDGET_CLASS (parent_class)->realize) {
                 GTK_WIDGET_CLASS (parent_class)->realize (widget);
         }
@@ -355,9 +326,7 @@ gs_window_real_realize (GtkWidget *widget)
 static gboolean
 watchdog_timer (GSWindow *window)
 {
-        GtkWidget *widget = GTK_WIDGET (window);
-
-        gdk_window_focus (widget->window, GDK_CURRENT_TIME);
+        gtk_window_present (GTK_WINDOW (window));
 
         return TRUE;
 }
@@ -395,6 +364,21 @@ add_popup_dialog_idle (GSWindow *window)
         window->priv->popup_dialog_idle_id = g_idle_add ((GSourceFunc)popup_dialog_idle, window);
 }
 
+static void
+remove_request_unlock_idle (GSWindow *window)
+{
+        if (window->priv->request_unlock_idle_id != 0) {
+                g_source_remove (window->priv->request_unlock_idle_id);
+                window->priv->request_unlock_idle_id = 0;
+        }
+}
+
+static void
+add_request_unlock_idle (GSWindow *window)
+{
+        window->priv->request_unlock_idle_id = g_idle_add ((GSourceFunc)gs_window_request_unlock_idle, window);
+}
+
 static gboolean
 emit_deactivated_idle (GSWindow *window)
 {
@@ -496,10 +480,14 @@ select_popup_events (void)
         XWindowAttributes attr;
         unsigned long     events;
 
+        gdk_error_trap_push ();
+
         XGetWindowAttributes (GDK_DISPLAY (), GDK_ROOT_WINDOW (), &attr);
 
         events = SubstructureNotifyMask | attr.your_event_mask;
-        XSelectInput (GDK_DISPLAY (), GDK_ROOT_WINDOW(), events);
+        XSelectInput (GDK_DISPLAY (), GDK_ROOT_WINDOW (), events);
+        gdk_flush ();
+        gdk_error_trap_pop ();
 }
 
 static void
@@ -582,8 +570,7 @@ get_env_vars (GtkWidget *widget)
                 "KRB5CCNAME",
                 "KRBTKFILE",
                 "LANG",
-                "LANGUAGE",
-                "RUNNING_UNDER_GDM"
+                "LANGUAGE"
         };
 
         env = g_ptr_array_new ();
@@ -1017,11 +1004,6 @@ gs_window_request_unlock (GSWindow *window)
 
         gs_debug ("Requesting unlock");
 
-        if (! GTK_WIDGET_VISIBLE (GTK_WIDGET (window))) {
-                gs_debug ("Request unlock but window is not visible!");
-                return;
-        }
-
         if (window->priv->watch_id > 0) {
                 return;
         }
@@ -1201,24 +1183,30 @@ gs_window_get_property (GObject    *object,
         case PROP_MONITOR:
                 g_value_set_int (value, self->priv->monitor);
                 break;
-        case PROP_OBSCURED:
-                g_value_set_boolean (value, self->priv->obscured);
-                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
         }
 }
 
+static gboolean
+gs_window_request_unlock_idle (GSWindow *window)
+{
+        gs_window_request_unlock (window);
+
+        window->priv->request_unlock_idle_id = 0;
+
+        return FALSE;
+}
+
 static void
 queue_key_event (GSWindow    *window,
                  GdkEventKey *event)
 {
-        /* Eat the first return, enter, escape, or space */
+        /* Eat the first return, enter, or space */
         if (window->priv->key_events == NULL
             && (event->keyval == GDK_Return
                 || event->keyval == GDK_KP_Enter
-                || event->keyval == GDK_Escape
                 || event->keyval == GDK_space)) {
                 return;
         }
@@ -1227,22 +1215,6 @@ queue_key_event (GSWindow    *window,
                                                    gdk_event_copy ((GdkEvent *)event));
 }
 
-static gboolean
-maybe_handle_activity (GSWindow *window)
-{
-        gboolean handled;
-
-        handled = FALSE;
-
-        /* if we already have a socket then don't bother */
-        if (! window->priv->socket
-            && GTK_WIDGET_IS_SENSITIVE (GTK_WIDGET (window))) {
-                g_signal_emit (window, signals [ACTIVITY], 0, &handled);
-        }
-
-        return handled;
-}
-
 static gboolean
 gs_window_real_key_press_event (GtkWidget   *widget,
                                 GdkEventKey *event)
@@ -1251,7 +1223,18 @@ gs_window_real_key_press_event (GtkWidget   *widget,
 
         /*g_message ("KEY PRESS state: %u keyval %u", event->state, event->keyval);*/
 
-        catch_events = maybe_handle_activity (GS_WINDOW (widget));
+        /* if we don't already have a socket then request an unlock */
+        if (! GS_WINDOW (widget)->priv->socket) {
+                if (GS_WINDOW (widget)->priv->request_unlock_idle_id == 0) {
+                        add_request_unlock_idle (GS_WINDOW (widget));
+                }
+
+                catch_events = TRUE;
+        } else {
+                if (! GTK_WIDGET_VISIBLE (GS_WINDOW (widget)->priv->socket)) {
+                        catch_events = TRUE;
+                }
+        }
 
         /* Catch all keypresses up until the lock dialog is shown */
         if (catch_events) {
@@ -1288,8 +1271,11 @@ gs_window_real_motion_notify_event (GtkWidget      *widget,
                         ABS (window->priv->last_y - event->y));
 
         if (distance > min_distance) {
-                maybe_handle_activity (window);
-
+                /* if we don't already have a socket then request an unlock */
+                if (! window->priv->socket
+                    && (window->priv->request_unlock_idle_id == 0)) {
+                        add_request_unlock_idle (window);
+                }
                 window->priv->last_x = -1;
                 window->priv->last_y = -1;
         }
@@ -1297,30 +1283,6 @@ gs_window_real_motion_notify_event (GtkWidget      *widget,
         return FALSE;
 }
 
-static gboolean
-gs_window_real_button_press_event (GtkWidget      *widget,
-                                   GdkEventButton *event)
-{
-        GSWindow *window;
-
-        window = GS_WINDOW (widget);
-        maybe_handle_activity (window);
-
-        return FALSE;
-}
-
-static gboolean
-gs_window_real_scroll_event (GtkWidget      *widget,
-                             GdkEventScroll *event)
-{
-        GSWindow *window;
-
-        window = GS_WINDOW (widget);
-        maybe_handle_activity (window);
-
-        return FALSE;
-}
-
 static void
 gs_window_real_size_request (GtkWidget      *widget,
                              GtkRequisition *requisition)
@@ -1380,46 +1342,6 @@ gs_window_real_grab_broken (GtkWidget          *widget,
         return FALSE;
 }
 
-gboolean
-gs_window_is_obscured (GSWindow *window)
-{
-        g_return_val_if_fail (GS_IS_WINDOW (window), FALSE);
-
-        return window->priv->obscured;
-}
-
-static void
-window_set_obscured (GSWindow *window,
-                     gboolean  obscured)
-{
-        if (window->priv->obscured == obscured) {
-                return;
-        }
-
-        window->priv->obscured = obscured;
-        g_object_notify (G_OBJECT (window), "obscured");
-}
-
-static gboolean
-gs_window_real_visibility_notify_event (GtkWidget          *widget,
-                                        GdkEventVisibility *event)
-{
-        switch (event->state) {
-        case GDK_VISIBILITY_FULLY_OBSCURED:
-                window_set_obscured (GS_WINDOW (widget), TRUE);
-                break;
-        case GDK_VISIBILITY_PARTIAL:
-                break;
-        case GDK_VISIBILITY_UNOBSCURED:
-                window_set_obscured (GS_WINDOW (widget), FALSE);
-                break;
-        default:
-                break;
-        }
-
-        return FALSE;
-}
-
 static void
 gs_window_class_init (GSWindowClass *klass)
 {
@@ -1438,24 +1360,11 @@ gs_window_class_init (GSWindowClass *klass)
         widget_class->unrealize           = gs_window_real_unrealize;
         widget_class->key_press_event     = gs_window_real_key_press_event;
         widget_class->motion_notify_event = gs_window_real_motion_notify_event;
-        widget_class->button_press_event  = gs_window_real_button_press_event;
-        widget_class->scroll_event        = gs_window_real_scroll_event;
         widget_class->size_request        = gs_window_real_size_request;
         widget_class->grab_broken_event   = gs_window_real_grab_broken;
-        widget_class->visibility_notify_event = gs_window_real_visibility_notify_event;
 
         g_type_class_add_private (klass, sizeof (GSWindowPrivate));
 
-        signals [ACTIVITY] =
-                g_signal_new ("activity",
-                              G_TYPE_FROM_CLASS (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GSWindowClass, activity),
-                              NULL,
-                              NULL,
-                              gs_marshal_BOOLEAN__VOID,
-                              G_TYPE_BOOLEAN,
-                              0);
         signals [DEACTIVATED] =
                 g_signal_new ("deactivated",
                               G_TYPE_FROM_CLASS (object_class),
@@ -1487,13 +1396,6 @@ gs_window_class_init (GSWindowClass *klass)
                               G_TYPE_NONE,
                               0);
 
-        g_object_class_install_property (object_class,
-                                         PROP_OBSCURED,
-                                         g_param_spec_boolean ("obscured",
-                                                               NULL,
-                                                               NULL,
-                                                               FALSE,
-                                                               G_PARAM_READABLE));
         g_object_class_install_property (object_class,
                                          PROP_LOCK_ENABLED,
                                          g_param_spec_boolean ("lock-enabled",
@@ -1518,7 +1420,7 @@ gs_window_class_init (GSWindowClass *klass)
                                                             0,
                                                             G_PARAM_READWRITE));
         g_object_class_install_property (object_class,
-                                         PROP_LOGOUT_COMMAND,
+                                         PROP_LOGOUT_TIMEOUT,
                                          g_param_spec_string ("logout-command",
                                                               NULL,
                                                               NULL,
@@ -1567,10 +1469,8 @@ gs_window_init (GSWindow *window)
                                | GDK_KEY_PRESS_MASK
                                | GDK_KEY_RELEASE_MASK
                                | GDK_EXPOSURE_MASK
-                               | GDK_VISIBILITY_NOTIFY_MASK
                                | GDK_ENTER_NOTIFY_MASK
                                | GDK_LEAVE_NOTIFY_MASK);
-        force_no_pixmap_background (GTK_WIDGET (window));
 }
 
 static void
@@ -1597,6 +1497,7 @@ gs_window_finalize (GObject *object)
         g_free (window->priv->logout_command);
 
         remove_watchdog_timer (window);
+        remove_request_unlock_idle (window);
         remove_popup_dialog_idle (window);
 
         if (window->priv->timer) {
